#!/usr/bin/perl

# Perl program to save an read a PGM image
#
#
# Fraga, April 16, 2001
#	  Comments Added, May 14, 2003
#	  Comments Added, May 14, 2004
#	  Function read_pgm_img Added, May 27, 2004

die "Args: image_file\n" if ($#ARGV!=0);

$file = $ARGV[0];

print "File: $file\n";

$r = read_pgm_img( $file );
if ( $r ) {
	print "Error reading image!\n";
}
save_pgm_img (\@myimg, 100, 100, "test.pgm");


# Subroutine to save an image in raw PGM format
# (see pgm man pages on NETPBM package)
sub save_pgm_img (\@$$$) {  # ( @img, $rows, $cols, $filename );
	local (*myimg) = shift(@_);
	my $myrows = shift(@_);
	my $mycols = shift(@_);
	my $filename = shift(@_);

	my $n = @myimg;
	printf "$n $mycols $myrows $filename\n";

	open ( MYFILE, ">$filename" ) or return 1;

	select MYFILE;
	$| = 1;
	print MYFILE "P5\n";
	print MYFILE "# file: $filename\n";
	print MYFILE "$mycols $myrows\n";
	print MYFILE "255\n";

	my @data;
	for ( my $i=0; $i<$myrows; $i++ ){
		for ( my $j=0; $j<$mycols; $j++ ){
			$data[$j] = $myimg[ $mycols*$i + $j ];
		}

		my $bytes = pack( "C$mycols", @data );
		syswrite ( MYFILE, $bytes, $mycols );
	}

	close MYFILE;
	select STDOUT;
	return 0;
}

sub get_int (*) {   
	my $fh = shift;
	
	my $c = getc( $fh );
	while ( 1 ){
		if ( $c eq '#' ){
			while( $c != '\n' ) {
				$c = getc( $fh );
				if ( $c eq "" ){ return 0;}
			} 
		}
		if ( $c ge '0' && $c le '9' ){
			last;
		}
		$c = getc( $fh );
	}
	my $i = 0;
	while (1) {
		$i = ($i*10) + ( $c - '0');
		$c = getc( $fh );
		if ($c eq ""){ return 0; }
		if ($c lt '0' || $c gt '9'){
			last;
		} 
	}
	return $i;
}  

sub read_pgm_img($)
{
	my $filename = shift;

	open ( MYFILE, "<$filename" ) or return 1;


	my $c1 = getc( MYFILE );
	my $c2 = getc( MYFILE );
	# print ( "HEADER: $c1 $c2\n" );
	if ( $c1 != 'P' && ( $c2!='4' || $c2!='5' ) ) {
		print (  "ERROR: Format no supported\n" );
		return 1;
	}
	my $mytype =  ( $c2=='4' ) ? 1 : 5;

	$mycols = get_int( \*MYFILE );
	if ( $mycols <= 0 ){ return 1; }
	$myrows = get_int( \*MYFILE );
	if ( $myrows <= 0 ){ return 1; }

	if ( $mytype == 5 ){ $greytones = get_int( \*MYFILE ); }

	print "$myrows $mycols $greytones\n";
	select MYFILE;
	$| = 1;
	for ( my $i=0; $i<$myrows; $i++ ){
		$bytes = 0;
		my $n = read ( MYFILE, $bytes, $mycols, 0 );
		if ( $n!=$mycols ) {
			close MYFILE;
			return 1;
		}
		my @data = unpack( "C$mycols", $bytes );
		for ( my $j=0; $j<$mycols; $j++ ){
			$myimg[ $mycols*$i + $j ] = $data[$j]; 
		}

	}
	close MYFILE;

	return 0;
}



