#!/usr/bin/perl

# Rotation of an image
#
# 21/May/2004

# A test image of size 256x256 pixels
$rows = 256;
$cols = 256;

# Limpio la imagen
for( $i=0; $i<$rows; $i++ ){
	for( $j=0; $j<$cols; $j++ ){
		$imgA[ $i * $cols + $j ] = 255;
		$imgB[ $i * $cols + $j ] = 0;
	}
}
# Ponemos un rectngulo blanco en el centro
$c=40;  # Largo
$d=100; # Ancho
for( $i=$c; $i<$rows-$c; $i++ ){
	for( $j=$d; $j<$cols-$d; $j++ ){
		$imgA[ $i * $cols + $j ] = 0;
	}
}
# Salvamos la imagen original
$r = save_pgm_img ( \@imgA, $rows, $cols, "rect.pgm" );

# Vamos a rotar la imagen

$theta = 30; # En grados

rotate( \@imgA, \@imgB, $rows, $cols, $theta );

$r = save_pgm_img ( \@imgB, $rows, $cols, "rot.pgm" );
if ( $r ) {
	print "Error saving image!\n";
}

#################################
# Subroutine to rotate an image
#
sub rotate (\@\@$$$) {  # ( @imgIN, @imgOUT, $rows, $cols, $ang );
	local (*imgIN) = shift(@_);
	local (*imgOUT) = shift(@_);
	my $myrows = shift(@_);
	my $mycols = shift(@_);
	my $ang = shift(@_);

	my $lcos = cos( $ang*3.141592654/180 );
	my $lsin = sin( $ang*3.141592654/180 );

	$cx = ($cols - 1)/2;
	$cy = ($rows - 1)/2;

	my $xi = -$cx*$lcos  - $cy*$lsin;
	my $yi =  $cx*$lsin  - $cy*$lcos;

	for ( $i=0; $i<$myrows; $i++ ) {
		my $x = $xi;
		my $y = $yi;
		for ( $j=0; $j<$mycols; $j++ ) {
			# Accesamos los valores de la imagen original

			my $intx = int ( $x + $cx + 0.5 );
			my $inty = int ( $y + $cy + 0.5 );
			
			if ( $intx>=0 && $intx<$mycols ) {
				if ( $inty>=0 && $inty<$myrows ) {
					$imgOUT[ $i + $mycols*$j ] =
						$imgIN[ $intx + $mycols*$inty ];
				}
			}

			$x += $lcos;
			$y -= $lsin;
		}
		$xi += $lsin;
		$yi += $lcos;
	}
	
}


# Subroutine to save an image in raw PGM format
# (see pgm man pages on NETPBM package)
sub save_pgm_img (\@$$$) {  # ( @img, $rows, $cols, $filename );
	local (*myimg) = shift(@_);
	my $myrows = shift(@_);
	my $mycols = shift(@_);
	my $filename = shift(@_);

	my $n = @myimg;
	printf "$n $mycols $myrows $filename\n";

	open ( MYFILE, ">$filename" ) or return 1;

	select MYFILE;
	$| = 1;
	print MYFILE "P5\n";
	print MYFILE "# file: $filename\n";
	print MYFILE "$mycols $myrows\n";
	print MYFILE "255\n";

	my @data;
	for ( my $i=0; $i<$myrows; $i++ ){
		for ( my $j=0; $j<$mycols; $j++ ){
			$data[$j] = $myimg[ $mycols*$i + $j ];
		}

		my $bytes = pack( "C$mycols", @data );
		syswrite ( MYFILE, $bytes, $n );
	}

	close MYFILE;
	select STDOUT;
	return 0;
}
